/*	Sample program for Olimex AVR-P40-8515 with AT90S8515 processor
 *	Echoes back the received characters on the UART. In order to work,
 *	connect the RX pad with PD1(pin 11) and TX pad with PD0(pin 10)
 *	Compile with AVRStudio+WinAVR (gcc version 3.4.6)
 */

#define	__AVR_AT90S8515__	1
#define OSCSPEED	4000000		/* in Hz */

#include "avr/io.h"

void PORT_Init()
{
	PORTB = 0b00000001;		//Turn OFF the led (Bit0 = 1)
	DDRB  = 0b00000001;		//set Led as output

	PORTD = 0b00000000;
	DDRD  = 0b00000010;		//set TX as output
}

void  UART_Init(uint32_t Baud)
{
	unsigned int BaudRate = OSCSPEED / (16 * Baud) - 1;		//calculate BaudRate

	//set BaudRate into register
	UBRR = (unsigned char) BaudRate;

	UCR = UCR | 0b00011000;		//enable Receiver (Bit4 = 1) and Transmitter (Bit3 = 1)
}

unsigned char UART_Receive()
{
	if (USR & 0b10000000)
		return UDR;
	else
		return 0;
}

void UART_Transmit(unsigned char Data)
{
	while (!(USR & 0b00100000));
	UDR = Data;
}

int main()
{
	unsigned char Ch=0;
	PORT_Init();
	UART_Init(9600);
	while (1)
	{
		Ch = UART_Receive();
		if (Ch)
		{
			UART_Transmit(Ch);
		}
	}
	return 0;
}
